@@ -10,8 +10,12 @@
 *
 */
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include "app-chooser-dialog.h"
-#include "app-chooser-dialog-ui.h"
+#include "ptk-ui-xml.h"
 
 #include <gtk/gtk.h>
 #include <glib.h>
@@ -79,7 +83,7 @@ static void add_list_item( GtkListStore* list, VFSAppDesktop* desktop )
     GtkTreeIter it;
     GdkPixbuf* icon = NULL;
 
-    icon = vfs_app_desktop_get_icon( desktop, 20 );
+    icon = vfs_app_desktop_get_icon( desktop, 20, TRUE );
     gtk_list_store_append( list, &it );
     gtk_list_store_set( list, &it, COL_APP_ICON, icon,
                         COL_APP_NAME, vfs_app_desktop_get_disp_name( desktop ),
@@ -90,7 +94,8 @@ static void add_list_item( GtkListStore* list, VFSAppDesktop* desktop )
 
 static GtkTreeModel* create_model_from_mime_type( VFSMimeType* mime_type )
 {
-    char** apps, **app, *type;
+    char** apps, **app;
+    const char *type;
     GtkListStore* list = gtk_list_store_new( N_COLS, GDK_TYPE_PIXBUF,
                                                     G_TYPE_STRING, G_TYPE_STRING );
     if ( mime_type )
@@ -114,13 +119,13 @@ static GtkTreeModel* create_model_from_mime_type( VFSMimeType* mime_type )
             g_strfreev( apps );
         }
     }
-    return list;
+    return (GtkTreeModel*) list;
 }
 
 GtkWidget* app_chooser_dialog_new( GtkWindow* parent, VFSMimeType* mime_type )
 {
-    GtkWidget * dlg = create_app_chooser_dlg();
-    GtkWidget* file_type = g_object_get_data( G_OBJECT(dlg), "file_type" );
+    GtkWidget * dlg = ptk_ui_xml_create_widget_from_file( PACKAGE_UI_DIR "/appchooserdlg.glade" );
+    GtkWidget* file_type = ptk_ui_xml_get_widget( dlg, "file_type" );
     const char* mime_desc;
     GtkTreeView* view;
     GtkTreeModel* model;
@@ -133,10 +138,10 @@ GtkWidget* app_chooser_dialog_new( GtkWindow* parent, VFSMimeType* mime_type )
     if ( 0 == strcmp( vfs_mime_type_get_type( mime_type ), XDG_MIME_TYPE_UNKNOWN ) ||
          0 == strcmp( vfs_mime_type_get_type( mime_type ), XDG_MIME_TYPE_DIRECTORY ) )
     {
-        gtk_widget_hide( g_object_get_data( G_OBJECT(dlg), "set_default" ) );
+        gtk_widget_hide( ptk_ui_xml_get_widget( dlg, "set_default" ) );
     }
 
-    view = GTK_TREE_VIEW( g_object_get_data( G_OBJECT(dlg), "recommended_apps" ) );
+    view = GTK_TREE_VIEW( ptk_ui_xml_get_widget( dlg, "recommended_apps" ) );
 
     model = create_model_from_mime_type( mime_type );
     gtk_tree_view_set_model( view, model );
@@ -144,6 +149,13 @@ GtkWidget* app_chooser_dialog_new( GtkWindow* parent, VFSMimeType* mime_type )
     init_list_view( view );
     gtk_widget_grab_focus( GTK_WIDGET( view ) );
 
+    g_signal_connect( ptk_ui_xml_get_widget( dlg, "notebook"),
+                                    "switch_page",
+                                    G_CALLBACK(on_notebook_switch_page), dlg );
+    g_signal_connect( ptk_ui_xml_get_widget( dlg, "browse_btn"),
+                                    "clicked",
+                                    G_CALLBACK(on_browse_btn_clicked), dlg );
+
     gtk_window_set_transient_for( GTK_WINDOW( dlg ), parent );
     return dlg;
 }
@@ -160,7 +172,7 @@ static void on_load_all_apps_finish( VFSAsyncTask* task, gboolean is_cancelled,
         return;
     }
 
-    view = (GtkTreeView*)g_object_get_data( task, "view" );
+    view = (GtkTreeView*)g_object_get_data( G_OBJECT(task), "view" );
 
     gtk_tree_sortable_set_sort_func ( GTK_TREE_SORTABLE( model ),
                                     COL_APP_NAME, sort_by_name, NULL, NULL );
@@ -179,13 +191,13 @@ on_notebook_switch_page ( GtkNotebook *notebook,
                           guint page_num,
                           gpointer user_data )
 {
-    GtkWidget * dlg = ( GtkWidget* ) notebook;
+    GtkWidget * dlg = ( GtkWidget* ) user_data;
     GtkTreeView* view;
 
     /* Load all known apps installed on the system */
     if ( page_num == 1 )
     {
-        view = GTK_TREE_VIEW( g_object_get_data( G_OBJECT(dlg), "all_apps" ) );
+        view = GTK_TREE_VIEW( ptk_ui_xml_get_widget( dlg, "all_apps" ) );
         if ( ! gtk_tree_view_get_model( view ) )
         {
             GdkCursor* busy;
@@ -194,14 +206,14 @@ on_notebook_switch_page ( GtkNotebook *notebook,
             init_list_view( view );
             gtk_widget_grab_focus( GTK_WIDGET( view ) );
             busy = gdk_cursor_new_for_display( gtk_widget_get_display(GTK_WIDGET( view )), GDK_WATCH );
-            gdk_window_set_cursor( GTK_WIDGET( gtk_widget_get_toplevel(view) )->window, busy );
+            gdk_window_set_cursor( GTK_WIDGET( gtk_widget_get_toplevel(GTK_WIDGET(view)) )->window, busy );
             gdk_cursor_unref( busy );
 
             list = gtk_list_store_new( N_COLS, GDK_TYPE_PIXBUF,
                                        G_TYPE_STRING, G_TYPE_STRING );
-            task = vfs_async_task_new( load_all_known_apps_thread, list );
-            g_object_set_data( task, "view", view );
-            g_object_set_data( dlg, "task", task );
+            task = vfs_async_task_new( (VFSAsyncFunc) load_all_known_apps_thread, list );
+            g_object_set_data( G_OBJECT(task), "view", view );
+            g_object_set_data( G_OBJECT(dlg), "task", task );
             g_signal_connect( task, "finish", G_CALLBACK(on_load_all_apps_finish), dlg );
             vfs_async_task_execute( task );
         }
@@ -217,7 +229,7 @@ on_notebook_switch_page ( GtkNotebook *notebook,
 const gchar* app_chooser_dialog_get_selected_app( GtkWidget* dlg )
 {
     const gchar * app = NULL;
-    GtkEntry* entry = GTK_ENTRY( g_object_get_data( G_OBJECT(dlg), "cmdline" ) );
+    GtkEntry* entry = GTK_ENTRY( ptk_ui_xml_get_widget( dlg, "cmdline" ) );
     GtkNotebook* notebook;
     int idx;
     GtkBin* scroll;
@@ -232,7 +244,7 @@ const gchar* app_chooser_dialog_get_selected_app( GtkWidget* dlg )
         return g_strdup( app );
     }
 
-    notebook = GTK_NOTEBOOK( g_object_get_data( G_OBJECT(dlg), "notebook" ) );
+    notebook = GTK_NOTEBOOK( ptk_ui_xml_get_widget( dlg, "notebook" ) );
     idx = gtk_notebook_get_current_page ( notebook );
     scroll = GTK_BIN( gtk_notebook_get_nth_page( notebook, idx ) );
     view = GTK_TREE_VIEW(gtk_bin_get_child( scroll ));
@@ -252,7 +264,7 @@ const gchar* app_chooser_dialog_get_selected_app( GtkWidget* dlg )
 */
 gboolean app_chooser_dialog_get_set_default( GtkWidget* dlg )
 {
-    GtkWidget * check = g_object_get_data( G_OBJECT(dlg), "set_default" );
+    GtkWidget * check = ptk_ui_xml_get_widget( dlg, "set_default" );
     return gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON( check ) );
 }
 
@@ -265,7 +277,7 @@ on_browse_btn_clicked ( GtkButton *button,
     GtkEntry* entry;
     const char* app_path = "/usr/share/applications";
 
-    GtkWidget* parent = GTK_WIDGET( button );
+    GtkWidget* parent = GTK_WIDGET(user_data );
     GtkWidget* dlg = gtk_file_chooser_dialog_new( NULL, GTK_WINDOW( parent ),
                                                   GTK_FILE_CHOOSER_ACTION_OPEN,
                                                   GTK_STOCK_OPEN,
@@ -280,7 +292,7 @@ on_browse_btn_clicked ( GtkButton *button,
         filename = gtk_file_chooser_get_filename ( GTK_FILE_CHOOSER ( dlg ) );
         if( filename )
         {
-            entry = GTK_ENTRY( g_object_get_data( G_OBJECT(dlg), "cmdline" ) );
+            entry = GTK_ENTRY( ptk_ui_xml_get_widget( dlg, "cmdline" ) );
             /* FIXME: path shouldn't be hard-coded */
             if( g_str_has_prefix( filename, app_path )
                 && g_str_has_suffix( filename, ".desktop" ) )
@@ -308,7 +320,7 @@ static void on_dlg_response( GtkDialog* dlg, int id, gpointer user_data )
     case GTK_RESPONSE_NONE:
     case GTK_RESPONSE_DELETE_EVENT:
         /* cancel app loading on dialog closing... */
-        task = (VFSAsyncTask*)g_object_get_data( dlg, "task" );
+        task = (VFSAsyncTask*)g_object_get_data( G_OBJECT(dlg), "task" );
         if( task )
         {
             vfs_async_task_cancel( task );
@@ -356,7 +368,8 @@ void load_all_apps_in_dir( const char* dir_path, GtkListStore* list, VFSAsyncTas
     GDir* dir = g_dir_open( dir_path, 0, NULL );
     if( dir )
     {
-        char* name, *path;
+        const char* name;
+        char* path;
         VFSAppDesktop* app;
         while( (name = g_dir_read_name( dir )) )
         {
@@ -402,7 +415,7 @@ void load_all_apps_in_dir( const char* dir_path, GtkListStore* list, VFSAsyncTas
 
 gpointer load_all_known_apps_thread( VFSAsyncTask* task )
 {
-    char* dir, **dirs;
+    gchar* dir, **dirs;
     GtkListStore* list;
     gboolean cancel = FALSE;
 
@@ -414,7 +427,7 @@ gpointer load_all_known_apps_thread( VFSAsyncTask* task )
     load_all_apps_in_dir( dir, list, task );
     g_free( dir );
 
-    for( dirs = g_get_system_data_dirs(); ! task->cancel && *dirs; ++dirs )
+    for( dirs = (gchar **) g_get_system_data_dirs(); ! task->cancel && *dirs; ++dirs )
     {
         dir = g_build_filename( *dirs, "applications", NULL );
         load_all_apps_in_dir( dir, list, task );
