@@ -21,90 +21,17 @@
 #include <string.h>
 
 #include "pref-dialog.h"
-#include "pref-dialog-ui.h"
-
 #include "settings.h"
-
-#if 0
-static const char theme_dir[] = "/usr/share/icons";
-
-gboolean is_theme_compatible_with_gnome( const char* theme_name )
-{
-    gboolean ret = FALSE;
-    char* theme_path;
-    char* sub_dir_name;
-    char* sub_dir_path;
-    char* dir_icon_path;
-    GDir* sub_dir;
-    const char* icons[] =
-        { "gnome-fs-directory.png",
-          "gnome-fs-directory.xpm",
-          "gnome-fs-directory.svg"
-        };
-    int i;
-
-    theme_path = g_build_filename( theme_dir,
-                                   theme_name, NULL );
-    if ( ( sub_dir = g_dir_open( theme_path, NULL, NULL ) ) )
-    {
-        while ( !ret && ( sub_dir_name = g_dir_read_name( sub_dir ) ) )
-        {
-            sub_dir_path = g_build_filename( theme_path, sub_dir_name, NULL );
-            for ( i = 0; !ret && i < G_N_ELEMENTS( icons ); ++i )
-            {
-                dir_icon_path = g_build_filename( sub_dir_path,
-                                                  "filesystems",
-                                                  icons[ i ], NULL );
-                if ( g_file_test( dir_icon_path, G_FILE_TEST_EXISTS ) )
-                {
-                    ret = TRUE;
-                    break;
-                }
-                g_free( dir_icon_path );
-            }
-            g_free( sub_dir_path );
-        }
-        g_dir_close( sub_dir );
-    }
-    g_free( theme_path );
-    return ret;
-}
-
-void init_icon_theme_combo( GtkComboBox* icon_theme )
-{
-    GDir * dir;
-    int idx = 0, sel = -1;
-    const char* sel_name = appSettings.iconTheme ? appSettings.iconTheme : "gnome";
-    char* theme_name;
-    dir = g_dir_open( theme_dir, NULL, NULL );
-    if ( !dir )
-        return ;
-
-    while ( ( theme_name = g_dir_read_name( dir ) ) )
-    {
-        if ( is_theme_compatible_with_gnome( theme_name ) )
-        {
-            gtk_combo_box_append_text( icon_theme, theme_name );
-            if ( sel < 0 && 0 == strcmp( theme_name, sel_name ) )
-            {
-                sel = idx;
-            }
-            ++idx;
-        }
-    }
-    gtk_combo_box_set_active( icon_theme, sel );
-    g_dir_close( dir );
-}
-#endif
+#include "ptk-ui-xml.h"
 
 static void
 on_show_desktop_toggled( GtkToggleButton* show_desktop, GtkWidget* dlg )
 {
-    GtkWidget* desktop_page = g_object_get_data( G_OBJECT(dlg), "desktop_page" );
+    GtkWidget* desktop_page = ptk_ui_xml_get_widget( dlg,  "desktop_page" );
     gtk_container_foreach( GTK_CONTAINER(desktop_page),
-                           gtk_widget_set_sensitive,
-                           gtk_toggle_button_get_active( show_desktop ) );
-    gtk_widget_set_sensitive( show_desktop, TRUE );
+                           (GtkCallback) gtk_widget_set_sensitive,
+                           (gpointer) gtk_toggle_button_get_active( show_desktop ) );
+    gtk_widget_set_sensitive( GTK_WIDGET(show_desktop), TRUE );
 }
 
 gboolean show_preference_dialog( GtkWindow* parent )
@@ -114,7 +41,6 @@ gboolean show_preference_dialog( GtkWindow* parent )
     GtkWidget* bm_open_method;
     GtkWidget* max_thumb_size;
     GtkWidget* show_thumbnail;
-    GtkWidget* icon_theme;
     GtkWidget* terminal;
     GtkWidget* big_icon_size;
     GtkWidget* small_icon_size;
@@ -127,8 +53,6 @@ gboolean show_preference_dialog( GtkWindow* parent )
     GtkWidget* text_color;
 
     const char* filename_encoding;
-    char* theme_name;
-    gboolean theme_changed;
     int i;
     const int big_icon_sizes[] =
         {
@@ -141,6 +65,7 @@ gboolean show_preference_dialog( GtkWindow* parent )
     int ibig_icon = -1, ismall_icon = -1;
     const char* terminal_programs[] =
         {
+            "xfce4-terminal",
             "gnome-terminal",
             "konsole",
             "rxvt",
@@ -149,28 +74,22 @@ gboolean show_preference_dialog( GtkWindow* parent )
             "mlterm"
         };
 
-    GtkWidget* dlg = create_prefdlg();
+    GtkWidget* dlg = ptk_ui_xml_create_widget_from_file( PACKAGE_UI_DIR "/prefdlg.glade" );
     gtk_window_set_transient_for( GTK_WINDOW( dlg ), parent );
 
-    encoding = g_object_get_data( G_OBJECT(dlg), "filename_encoding" );
-    bm_open_method = g_object_get_data( G_OBJECT(dlg), "bm_open_method" );
-    show_thumbnail = g_object_get_data( G_OBJECT(dlg), "show_thumbnail" );
-    max_thumb_size = g_object_get_data( G_OBJECT(dlg), "max_thumb_size" );
-    icon_theme = g_object_get_data( G_OBJECT(dlg), "theme" );
-    terminal = g_object_get_data( G_OBJECT(dlg), "terminal" );
-    big_icon_size = g_object_get_data( G_OBJECT(dlg), "big_icon_size" );
-    small_icon_size = g_object_get_data( G_OBJECT(dlg), "small_icon_size" );
+    encoding = ptk_ui_xml_get_widget( dlg,  "filename_encoding" );
+    bm_open_method = ptk_ui_xml_get_widget( dlg,  "bm_open_method" );
+    show_thumbnail = ptk_ui_xml_get_widget( dlg,  "show_thumbnail" );
+    max_thumb_size = ptk_ui_xml_get_widget( dlg,  "max_thumb_size" );
+    terminal = ptk_ui_xml_get_widget( dlg,  "terminal" );
+    big_icon_size = ptk_ui_xml_get_widget( dlg,  "big_icon_size" );
+    small_icon_size = ptk_ui_xml_get_widget( dlg,  "small_icon_size" );
 
     if ( '\0' == ( char ) appSettings.encoding[ 0 ] )
         gtk_entry_set_text( GTK_ENTRY( encoding ), "UTF-8" );
     else
         gtk_entry_set_text( GTK_ENTRY( encoding ), appSettings.encoding );
 
-    /*
-    init_icon_theme_combo ( GTK_COMBO_BOX(icon_theme) );
-    */
-    gtk_widget_hide( gtk_widget_get_parent( icon_theme ) );
-
     if ( appSettings.openBookmarkMethod >= 1 &&
             appSettings.openBookmarkMethod <= 3 )
     {
@@ -181,6 +100,7 @@ gboolean show_preference_dialog( GtkWindow* parent )
     {
         gtk_combo_box_set_active( GTK_COMBO_BOX( bm_open_method ), 0 );
     }
+
     gtk_spin_button_set_value ( GTK_SPIN_BUTTON( max_thumb_size ),
                                 appSettings.maxThumbSize >> 10 );
 
@@ -227,24 +147,24 @@ gboolean show_preference_dialog( GtkWindow* parent )
     }
     gtk_combo_box_set_active( GTK_COMBO_BOX( small_icon_size ), ismall_icon );
 
-    show_desktop = g_object_get_data( G_OBJECT(dlg), "show_desktop" );
+    show_desktop = ptk_ui_xml_get_widget( dlg,  "show_desktop" );
     gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( show_desktop ),
                                   appSettings.showDesktop );
-    on_show_desktop_toggled( show_desktop, dlg );
+    on_show_desktop_toggled( GTK_TOGGLE_BUTTON(show_desktop), dlg );
     g_signal_connect( show_desktop, "toggled",
                       G_CALLBACK(on_show_desktop_toggled), dlg );
 
-    show_wallpaper = g_object_get_data( G_OBJECT(dlg), "show_wallpaper" );
+    show_wallpaper = ptk_ui_xml_get_widget( dlg,  "show_wallpaper" );
     gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( show_wallpaper ),
                                   appSettings.showWallpaper );
-    wallpaper = g_object_get_data( G_OBJECT(dlg), "wallpaper" );
+    wallpaper = ptk_ui_xml_get_widget( dlg,  "wallpaper" );
     if ( appSettings.wallpaper )
     {
         gtk_file_chooser_set_filename( GTK_FILE_CHOOSER( wallpaper ),
                                        appSettings.wallpaper );
     }
-    bg_color1 = g_object_get_data( G_OBJECT(dlg), "bg_color1" );
-    text_color = g_object_get_data( G_OBJECT(dlg), "text_color" );
+    bg_color1 = ptk_ui_xml_get_widget( dlg,  "bg_color1" );
+    text_color = ptk_ui_xml_get_widget( dlg,  "text_color" );
     gtk_color_button_set_color(GTK_COLOR_BUTTON(bg_color1),
                                &appSettings.desktopBg1);
     gtk_color_button_set_color(GTK_COLOR_BUTTON(text_color),
@@ -288,25 +208,6 @@ gboolean show_preference_dialog( GtkWindow* parent )
             appSettings.terminal = NULL;
         }
 
-#if 0
-        theme_name = gtk_combo_box_get_active_text( GTK_COMBO_BOX( icon_theme ) );
-        if ( ( theme_name && !appSettings.iconTheme ) ||
-                ( !theme_name && appSettings.iconTheme ) ||
-                strcmp( appSettings.iconTheme, theme_name ) )
-        {
-            theme_changed = TRUE;
-        }
-        if ( theme_changed )
-        {
-            if ( appSettings.iconTheme )
-                g_free( appSettings.iconTheme );
-            appSettings.iconTheme = theme_name;
-            /* set new theme */
-            g_object_set ( gtk_settings_get_default(),
-                           "gtk-icon-theme-name",
-                           theme_name ? theme_name : "gnome", NULL );
-        }
-#endif
         appSettings.showDesktop = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( show_desktop ) );
         appSettings.showWallpaper = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( show_wallpaper ) );
         g_free( appSettings.wallpaper );
