@@ -11,6 +11,7 @@
 */
 
 #include "vfs-dir.h"
+#include "vfs-thumbnail-loader.h"
 #include "glib-mem.h"
 
 #include <string.h>
@@ -30,7 +31,10 @@ static void vfs_dir_get_property ( GObject *obj,
                                    GValue *value,
                                    GParamSpec *pspec );
 
-static void vfs_dir_load( VFSDir* dir, const char* path );
+/* constructor is private */
+static VFSDir* vfs_dir_new( const char* path );
+
+static void vfs_dir_load( VFSDir* dir );
 static gpointer vfs_dir_load_thread( VFSAsyncTask* task, VFSDir* dir );
 
 static void vfs_dir_monitor_callback( VFSFileMonitor* fm,
@@ -38,7 +42,9 @@ static void vfs_dir_monitor_callback( VFSFileMonitor* fm,
                                       const char* file_name,
                                       gpointer user_data );
 
+#if 0
 static gpointer load_thumbnail_thread( gpointer user_data );
+#endif
 
 static void on_mime_type_reload( gpointer user_data );
 
@@ -48,7 +54,11 @@ static void update_changed_files( gpointer key, gpointer data,
 static gboolean notify_file_change( gpointer user_data );
 static gboolean update_file_info( VFSDir* dir, VFSFileInfo* file );
 
+#if 0
 static gboolean is_dir_desktop( const char* path );
+#endif
+
+static void on_list_task_finished( VFSAsyncTask* task, gboolean is_cancelled, VFSDir* dir );
 
 enum {
     FILE_CREATED_SIGNAL = 0,
@@ -65,6 +75,7 @@ static GObjectClass *parent_class = NULL;
 static GHashTable* dir_hash = NULL;
 static GList* mime_cb = NULL;
 static guint change_notify_timeout = 0;
+static guint theme_change_notify = 0;
 
 static char* desktop_dir = NULL;
 static gboolean is_desktop_set = FALSE;
@@ -178,13 +189,18 @@ void vfs_dir_init( VFSDir* dir )
 
 /* destructor */
 
-static void vfs_dir_clear( VFSDir* dir )
+void vfs_dir_finalize( GObject *obj )
 {
-    GList* l;
+    VFSDir * dir = VFS_DIR( obj );
+
+    do{}
+    while( g_source_remove_by_user_data( dir ) );
 
     if( G_UNLIKELY( dir->task ) )
     {
+        g_signal_handlers_disconnect_by_func( dir->task, on_list_task_finished, dir );
         /* FIXME: should we generate a "file-list" signal to indicate the dir loading was cancelled? */
+        vfs_async_task_cancel( dir->task );
         g_object_unref( dir->task );
         dir->task = NULL;
     }
@@ -214,6 +230,10 @@ static void vfs_dir_clear( VFSDir* dir )
                     g_source_remove( change_notify_timeout );
                     change_notify_timeout = 0;
                 }
+
+                g_signal_handler_disconnect( gtk_icon_theme_get_default(),
+                                                                theme_change_notify );
+                theme_change_notify = 0;
             }
         }
         g_free( dir->path );
@@ -242,15 +262,6 @@ static void vfs_dir_clear( VFSDir* dir )
         g_slist_free( dir->changed_files );
         dir->changed_files = NULL;
     }
-}
-
-void vfs_dir_finalize( GObject *obj )
-{
-    VFSDir * dir = VFS_DIR( obj );
-
-    do{}
-    while( g_source_remove_by_user_data( dir ) );
-    vfs_dir_clear( dir );
 
     g_mutex_free( dir->mutex );
     G_OBJECT_CLASS( parent_class ) ->finalize( obj );
@@ -330,8 +341,8 @@ void vfs_dir_emit_file_created( VFSDir* dir, const char* file_name, VFSFileInfo*
 
     if ( file )
     {
-        gboolean is_desktop = is_dir_desktop(dir->path);
-        if( G_UNLIKELY(is_desktop) && full_path )
+        /* gboolean is_desktop = is_dir_desktop(dir->path); */
+        if( /*G_UNLIKELY(is_desktop) &&*/ G_LIKELY(full_path) )
             vfs_file_info_load_special_info( file, full_path );
 
         dir->file_list = g_list_prepend( dir->file_list, vfs_file_info_ref(file) );
@@ -460,41 +471,38 @@ void vfs_dir_emit_thumbnail_loaded( VFSDir* dir, VFSFileInfo* file )
 
 /* methods */
 
-VFSDir* vfs_dir_new()
+VFSDir* vfs_dir_new( const char* path )
 {
     VFSDir * dir;
     dir = ( VFSDir* ) g_object_new( VFS_TYPE_DIR, NULL );
+    dir->path = g_strdup( path );
+    dir->disp_path = g_filename_display_name( path );
     return dir;
 }
 
-static void on_list_task_finished( VFSAsyncTask* task, gboolean is_cancelled, VFSDir* dir )
+void on_list_task_finished( VFSAsyncTask* task, gboolean is_cancelled, VFSDir* dir )
 {
     g_object_unref( dir->task );
     dir->task = NULL;
     g_signal_emit( dir, signals[FILE_LISTED_SIGNAL], 0, is_cancelled );
 }
 
-void vfs_dir_load( VFSDir* dir, const char* path )
+void vfs_dir_load( VFSDir* dir )
 {
-    GSList * l;
-    struct VFSDirStateCallbackEnt* ent;
-
-    if ( G_LIKELY(path) )
+    if ( G_LIKELY(dir->path) )
     {
-        vfs_dir_clear( dir );
-        dir->path = g_strdup( path );
-        dir->disp_path = g_filename_display_name( path );
-
         dir->task = vfs_async_task_new( (VFSAsyncFunc)vfs_dir_load_thread, dir );
         g_signal_connect( dir->task, "finish", G_CALLBACK(on_list_task_finished), dir );
         vfs_async_task_execute( dir->task );
     }
 }
 
+#if 0
 gboolean is_dir_desktop( const char* path )
 {
     return (desktop_dir && 0 == strcmp(path, desktop_dir));
 }
+#endif
 
 gpointer vfs_dir_load_thread(  VFSAsyncTask* task, VFSDir* dir )
 {
@@ -502,8 +510,7 @@ gpointer vfs_dir_load_thread(  VFSAsyncTask* task, VFSDir* dir )
     char* full_path;
     GDir* dir_content;
     VFSFileInfo* file;
-    GList* l;
-    gboolean is_desktop;
+    /* gboolean is_desktop; */
 
     dir->file_listed = 0;
     dir->load_complete = 0;
@@ -518,20 +525,22 @@ gpointer vfs_dir_load_thread(  VFSAsyncTask* task, VFSDir* dir )
         dir_content = g_dir_open( dir->path, 0, NULL );
         if ( dir_content )
         {
-            is_desktop = is_dir_desktop( dir->path );
+            /* is_desktop = is_dir_desktop( dir->path ); */
             while ( ! vfs_async_task_is_cancelled( dir->task )
                         && ( file_name = g_dir_read_name( dir_content ) ) )
             {
                 full_path = g_build_filename( dir->path, file_name, NULL );
                 if ( !full_path )
                     continue;
+                /* FIXME: Is locking GDK needed here? */
+                /* GDK_THREADS_ENTER(); */
                 file = vfs_file_info_new();
                 if ( G_LIKELY( vfs_file_info_get( file, full_path, file_name ) ) )
                 {
                     g_mutex_lock( dir->mutex );
                     /* Special processing for desktop folder */
-                    if( G_UNLIKELY(is_desktop) )
-                        vfs_file_info_load_special_info( file, full_path );
+                    /* if( G_UNLIKELY(is_desktop) ) */
+                    vfs_file_info_load_special_info( file, full_path );
                     dir->file_list = g_list_prepend( dir->file_list, file );
                     g_mutex_unlock( dir->mutex );
                     ++dir->n_files;
@@ -540,6 +549,7 @@ gpointer vfs_dir_load_thread(  VFSAsyncTask* task, VFSDir* dir )
                 {
                     vfs_file_info_unref( file );
                 }
+                /* GDK_THREADS_LEAVE(); */
                 g_free( full_path );
             }
             g_dir_close( dir_content );
@@ -575,8 +585,8 @@ gboolean update_file_info( VFSDir* dir, VFSFileInfo* file )
 {
     char* full_path;
     char* file_name;
-    gboolean ret;
-    gboolean is_desktop = is_dir_desktop(dir->path);
+    gboolean ret = FALSE;
+    /* gboolean is_desktop = is_dir_desktop(dir->path); */
 
     /* FIXME: Dirty hack: steal the string to prevent memory allocation */
     file_name = file->name;
@@ -591,8 +601,8 @@ gboolean update_file_info( VFSDir* dir, VFSFileInfo* file )
         if( G_LIKELY( vfs_file_info_get( file, full_path, file_name ) ) )
         {
             ret = TRUE;
-            if( G_UNLIKELY(is_desktop) )
-                vfs_file_info_load_special_info( file, full_path );
+            /* if( G_UNLIKELY(is_desktop) ) */
+            vfs_file_info_load_special_info( file, full_path );
         }
         else /* The file doesn't exist */
         {
@@ -674,14 +684,50 @@ void vfs_dir_monitor_callback( VFSFileMonitor* fm,
     GDK_THREADS_LEAVE();
 }
 
-VFSDir* vfs_get_dir( const char* path )
+/* called on every VFSDir when icon theme got changed */
+static void reload_icons( const char* path, VFSDir* dir, gpointer user_data )
+{
+    GList* l;
+    for( l = dir->file_list; l; l = l->next )
+    {
+        VFSFileInfo* fi = (VFSFileInfo*)l->data;
+        /* It's a desktop entry file */
+        if( fi->flags & VFS_FILE_INFO_DESKTOP_ENTRY )
+        {
+            char* file_path = g_build_filename( path, fi->name,NULL );
+            if( fi->big_thumbnail )
+            {
+                g_object_unref( fi->big_thumbnail );
+                fi->big_thumbnail = NULL;
+            }
+            if( fi->small_thumbnail )
+            {
+                g_object_unref( fi->small_thumbnail );
+                fi->small_thumbnail = NULL;
+            }
+            vfs_file_info_load_special_info( fi, file_path );
+            g_free( file_path );
+        }
+    }
+}
+
+static void on_theme_changed( GtkIconTheme *icon_theme, gpointer user_data )
+{
+    g_hash_table_foreach( dir_hash, (GHFunc)reload_icons, NULL );
+}
+
+VFSDir* vfs_dir_get_by_path( const char* path )
 {
     VFSDir * dir = NULL;
 
     g_return_val_if_fail( G_UNLIKELY(path), NULL );
 
     if ( G_UNLIKELY( ! dir_hash ) )
-        dir_hash = g_hash_table_new_full( g_str_hash, g_str_equal, NULL, g_object_unref );
+    {
+        dir_hash = g_hash_table_new_full( g_str_hash, g_str_equal, NULL, NULL );
+        theme_change_notify = g_signal_connect( gtk_icon_theme_get_default(), "changed",
+                                                                    G_CALLBACK( on_theme_changed ), NULL );
+    }
     else
         dir = g_hash_table_lookup( dir_hash, path );
 
@@ -689,14 +735,12 @@ VFSDir* vfs_get_dir( const char* path )
         mime_cb = vfs_mime_type_add_reload_cb( on_mime_type_reload, NULL );
 
     if ( dir )
-    {
         g_object_ref( dir );
-    }
     else
     {
-        dir = vfs_dir_new();
-        vfs_dir_load( dir, path );  /* asynchronous operation */
-        g_hash_table_insert( dir_hash, (gpointer)dir->path, (gpointer)g_object_ref(dir) );
+        dir = vfs_dir_new( path );
+        vfs_dir_load( dir );  /* asynchronous operation */
+        g_hash_table_insert( dir_hash, (gpointer)dir->path, (gpointer)dir );
     }
     return dir;
 }
@@ -768,7 +812,7 @@ const char* vfs_get_desktop_dir()
                     {
                         char* eol;
                         line += 16;
-                        if( G_LIKELY( eol = strchr( line, '\n' ) ) )
+                        if( G_LIKELY( ( eol = strchr( line, '\n' ) ) ) )
                             *eol = '\0';
                         line = g_shell_unquote( line, NULL );
                         if( g_str_has_prefix(line, "$HOME") )
@@ -814,6 +858,8 @@ void vfs_dir_unload_thumbnails( VFSDir* dir, gboolean is_big )
 {
     GList* l;
     VFSFileInfo* file;
+    char* file_path;
+
     g_mutex_lock( dir->mutex );
     if( is_big )
     {
@@ -825,6 +871,14 @@ void vfs_dir_unload_thumbnails( VFSDir* dir, gboolean is_big )
                 g_object_unref( file->big_thumbnail );
                 file->big_thumbnail = NULL;
             }
+            /* This is a desktop entry file, so the icon needs reload
+                 FIXME: This is not a good way to do things, but there is no better way now.  */
+            if( file->flags & VFS_FILE_INFO_DESKTOP_ENTRY )
+            {
+                file_path = g_build_filename( dir->path, file->name, NULL );
+                vfs_file_info_load_special_info( file, file_path );
+                g_free( file_path );
+            }
         }
     }
     else
@@ -837,6 +891,14 @@ void vfs_dir_unload_thumbnails( VFSDir* dir, gboolean is_big )
                 g_object_unref( file->small_thumbnail );
                 file->small_thumbnail = NULL;
             }
+            /* This is a desktop entry file, so the icon needs reload
+                 FIXME: This is not a good way to do things, but there is no better way now.  */
+            if( file->flags & VFS_FILE_INFO_DESKTOP_ENTRY )
+            {
+                file_path = g_build_filename( dir->path, file->name, NULL );
+                vfs_file_info_load_special_info( file, file_path );
+                g_free( file_path );
+            }
         }
     }
     g_mutex_unlock( dir->mutex );
