@@ -72,9 +72,8 @@ struct _VFSDirClass
 typedef void ( *VFSDirStateCallback ) ( VFSDir* dir, int state, gpointer user_data );
 
 GType vfs_dir_get_type ( void );
-VFSDir* vfs_dir_new();
 
-VFSDir* vfs_get_dir( const char* path );
+VFSDir* vfs_dir_get_by_path( const char* path );
 
 gboolean vfs_dir_is_loading( VFSDir* dir );
 void vfs_dir_cancel_load( VFSDir* dir );
